@@ -1,19 +1,23 @@
 /*********************************************************************
   Blosc - Blocked Shuffling and Compression Library
 
-  Author: Francesc Alted <francesc@blosc.org>
+  Copyright (c) 2021  Blosc Development Team <blosc@blosc.org>
+  https://blosc.org
+  License: BSD 3-Clause (see LICENSE.txt)
 
-  See LICENSES/BLOSC.txt for details about copyright and rights to use.
+  See LICENSE.txt for details about copyright and rights to use.
 **********************************************************************/
 
-#include "shuffle-generic.h"
 #include "shuffle-sse2.h"
+#include "shuffle-generic.h"
+#include <stdlib.h>
 
 /* Make sure SSE2 is available for the compilation target and compiler. */
 #if defined(__SSE2__)
 
 #include <emmintrin.h>
 
+#include <stdint.h>
 
 /* The next is useful for debugging purposes */
 #if 0
@@ -37,10 +41,9 @@ static void printxmm(__m128i xmm0)
 /* Routine optimized for shuffling a buffer for a type size of 2 bytes. */
 static void
 shuffle2_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 2;
-  size_t j;
+              const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 2;
+  int32_t j;
   int k;
   uint8_t* dest_for_jth_element;
   __m128i xmm0[2], xmm1[2];
@@ -61,8 +64,8 @@ shuffle2_sse2(uint8_t* const dest, const uint8_t* const src,
     }
     /* Transpose quad words */
     for (k = 0; k < 1; k++) {
-      xmm1[k*2] = _mm_unpacklo_epi64(xmm0[k], xmm0[k+1]);
-      xmm1[k*2+1] = _mm_unpackhi_epi64(xmm0[k], xmm0[k+1]);
+      xmm1[k * 2] = _mm_unpacklo_epi64(xmm0[k], xmm0[k + 1]);
+      xmm1[k * 2 + 1] = _mm_unpackhi_epi64(xmm0[k], xmm0[k + 1]);
     }
     /* Store the result vectors */
     dest_for_jth_element = dest + j;
@@ -75,10 +78,9 @@ shuffle2_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for shuffling a buffer for a type size of 4 bytes. */
 static void
 shuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 4;
-  size_t i;
+              const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 4;
+  int32_t i;
   int j;
   uint8_t* dest_for_ith_element;
   __m128i xmm0[4], xmm1[4];
@@ -95,13 +97,13 @@ shuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
     }
     /* Transpose double words */
     for (j = 0; j < 2; j++) {
-      xmm1[j*2] = _mm_unpacklo_epi32(xmm0[j*2], xmm0[j*2+1]);
-      xmm1[j*2+1] = _mm_unpackhi_epi32(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[j * 2] = _mm_unpacklo_epi32(xmm0[j * 2], xmm0[j * 2 + 1]);
+      xmm1[j * 2 + 1] = _mm_unpackhi_epi32(xmm0[j * 2], xmm0[j * 2 + 1]);
     }
     /* Transpose quad words */
     for (j = 0; j < 2; j++) {
-      xmm0[j*2] = _mm_unpacklo_epi64(xmm1[j], xmm1[j+2]);
-      xmm0[j*2+1] = _mm_unpackhi_epi64(xmm1[j], xmm1[j+2]);
+      xmm0[j * 2] = _mm_unpacklo_epi64(xmm1[j], xmm1[j + 2]);
+      xmm0[j * 2 + 1] = _mm_unpackhi_epi64(xmm1[j], xmm1[j + 2]);
     }
     /* Store the result vectors */
     dest_for_ith_element = dest + i;
@@ -114,10 +116,9 @@ shuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for shuffling a buffer for a type size of 8 bytes. */
 static void
 shuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 8;
-  size_t j;
+              const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 8;
+  int32_t j;
   int k, l;
   uint8_t* dest_for_jth_element;
   __m128i xmm0[8], xmm1[8];
@@ -130,20 +131,20 @@ shuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
       xmm1[k] = _mm_unpacklo_epi8(xmm0[k], xmm1[k]);
     }
     /* Transpose words */
-    for (k = 0, l = 0; k < 4; k++, l +=2) {
-      xmm0[k*2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l+1]);
-      xmm0[k*2+1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l+1]);
+    for (k = 0, l = 0; k < 4; k++, l += 2) {
+      xmm0[k * 2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l + 1]);
+      xmm0[k * 2 + 1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l + 1]);
     }
     /* Transpose double words */
     for (k = 0, l = 0; k < 4; k++, l++) {
       if (k == 2) l += 2;
-      xmm1[k*2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l+2]);
-      xmm1[k*2+1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l+2]);
+      xmm1[k * 2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l + 2]);
+      xmm1[k * 2 + 1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l + 2]);
     }
     /* Transpose quad words */
     for (k = 0; k < 4; k++) {
-      xmm0[k*2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k+4]);
-      xmm0[k*2+1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k+4]);
+      xmm0[k * 2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k + 4]);
+      xmm0[k * 2 + 1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k + 4]);
     }
     /* Store the result vectors */
     dest_for_jth_element = dest + j;
@@ -156,10 +157,9 @@ shuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for shuffling a buffer for a type size of 16 bytes. */
 static void
 shuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 16;
-  size_t j;
+               const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 16;
+  int32_t j;
   int k, l;
   uint8_t* dest_for_jth_element;
   __m128i xmm0[16], xmm1[16];
@@ -170,26 +170,26 @@ shuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
       xmm0[k] = _mm_loadu_si128((__m128i*)(src + (j * bytesoftype) + (k * sizeof(__m128i))));
     }
     /* Transpose bytes */
-    for (k = 0, l = 0; k < 8; k++, l +=2) {
-      xmm1[k*2] = _mm_unpacklo_epi8(xmm0[l], xmm0[l+1]);
-      xmm1[k*2+1] = _mm_unpackhi_epi8(xmm0[l], xmm0[l+1]);
+    for (k = 0, l = 0; k < 8; k++, l += 2) {
+      xmm1[k * 2] = _mm_unpacklo_epi8(xmm0[l], xmm0[l + 1]);
+      xmm1[k * 2 + 1] = _mm_unpackhi_epi8(xmm0[l], xmm0[l + 1]);
     }
     /* Transpose words */
     for (k = 0, l = -2; k < 8; k++, l++) {
-      if ((k%2) == 0) l += 2;
-      xmm0[k*2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l+2]);
-      xmm0[k*2+1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l+2]);
+      if ((k % 2) == 0) l += 2;
+      xmm0[k * 2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l + 2]);
+      xmm0[k * 2 + 1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l + 2]);
     }
     /* Transpose double words */
     for (k = 0, l = -4; k < 8; k++, l++) {
-      if ((k%4) == 0) l += 4;
-      xmm1[k*2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l+4]);
-      xmm1[k*2+1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l+4]);
+      if ((k % 4) == 0) l += 4;
+      xmm1[k * 2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l + 4]);
+      xmm1[k * 2 + 1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l + 4]);
     }
     /* Transpose quad words */
     for (k = 0; k < 8; k++) {
-      xmm0[k*2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k+8]);
-      xmm0[k*2+1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k+8]);
+      xmm0[k * 2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k + 8]);
+      xmm0[k * 2 + 1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k + 8]);
     }
     /* Store the result vectors */
     dest_for_jth_element = dest + j;
@@ -202,10 +202,9 @@ shuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for shuffling a buffer for a type size larger than 16 bytes. */
 static void
 shuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements, const size_t bytesoftype)
-{
-  size_t j;
-  const size_t vecs_per_el_rem = bytesoftype % sizeof(__m128i);
+                     const int32_t vectorizable_elements, const int32_t total_elements, const int32_t bytesoftype) {
+  int32_t j;
+  const int32_t vecs_per_el_rem = bytesoftype % (int32_t)sizeof(__m128i);
   int k, l;
   uint8_t* dest_for_jth_element;
   __m128i xmm0[16], xmm1[16];
@@ -215,9 +214,10 @@ shuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const src,
     the initial iteration and the type size is not a multiple of the vector size.
     In that case, only advance by the number of bytes necessary so that the number
     of remaining bytes in the type will be a multiple of the vector size. */
-    size_t offset_into_type;
+    int32_t offset_into_type;
     for (offset_into_type = 0; offset_into_type < bytesoftype;
-      offset_into_type += (offset_into_type == 0 && vecs_per_el_rem > 0 ? vecs_per_el_rem : sizeof(__m128i))) {
+         offset_into_type += (offset_into_type == 0 &&
+                              vecs_per_el_rem > 0 ? vecs_per_el_rem : (int32_t)sizeof(__m128i))) {
 
       /* Fetch elements in groups of 256 bytes */
       const uint8_t* const src_with_offset = src + offset_into_type;
@@ -225,26 +225,26 @@ shuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const src,
         xmm0[k] = _mm_loadu_si128((__m128i*)(src_with_offset + (j + k) * bytesoftype));
       }
       /* Transpose bytes */
-      for (k = 0, l = 0; k < 8; k++, l +=2) {
-        xmm1[k*2] = _mm_unpacklo_epi8(xmm0[l], xmm0[l+1]);
-        xmm1[k*2+1] = _mm_unpackhi_epi8(xmm0[l], xmm0[l+1]);
+      for (k = 0, l = 0; k < 8; k++, l += 2) {
+        xmm1[k * 2] = _mm_unpacklo_epi8(xmm0[l], xmm0[l + 1]);
+        xmm1[k * 2 + 1] = _mm_unpackhi_epi8(xmm0[l], xmm0[l + 1]);
       }
       /* Transpose words */
       for (k = 0, l = -2; k < 8; k++, l++) {
-        if ((k%2) == 0) l += 2;
-        xmm0[k*2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l+2]);
-        xmm0[k*2+1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l+2]);
+        if ((k % 2) == 0) l += 2;
+        xmm0[k * 2] = _mm_unpacklo_epi16(xmm1[l], xmm1[l + 2]);
+        xmm0[k * 2 + 1] = _mm_unpackhi_epi16(xmm1[l], xmm1[l + 2]);
       }
       /* Transpose double words */
       for (k = 0, l = -4; k < 8; k++, l++) {
-        if ((k%4) == 0) l += 4;
-        xmm1[k*2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l+4]);
-        xmm1[k*2+1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l+4]);
+        if ((k % 4) == 0) l += 4;
+        xmm1[k * 2] = _mm_unpacklo_epi32(xmm0[l], xmm0[l + 4]);
+        xmm1[k * 2 + 1] = _mm_unpackhi_epi32(xmm0[l], xmm0[l + 4]);
       }
       /* Transpose quad words */
       for (k = 0; k < 8; k++) {
-        xmm0[k*2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k+8]);
-        xmm0[k*2+1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k+8]);
+        xmm0[k * 2] = _mm_unpacklo_epi64(xmm1[k], xmm1[k + 8]);
+        xmm0[k * 2 + 1] = _mm_unpackhi_epi64(xmm1[k], xmm1[k + 8]);
       }
       /* Store the result vectors */
       dest_for_jth_element = dest + j;
@@ -258,10 +258,9 @@ shuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for unshuffling a buffer for a type size of 2 bytes. */
 static void
 unshuffle2_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 2;
-  size_t i;
+                const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 2;
+  int32_t i;
   int j;
   __m128i xmm0[2], xmm1[2];
 
@@ -285,10 +284,9 @@ unshuffle2_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for unshuffling a buffer for a type size of 4 bytes. */
 static void
 unshuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 4;
-  size_t i;
+                const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 4;
+  int32_t i;
   int j;
   __m128i xmm0[4], xmm1[4];
 
@@ -301,16 +299,16 @@ unshuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
     /* Shuffle bytes */
     for (j = 0; j < 2; j++) {
       /* Compute the low 32 bytes */
-      xmm1[j] = _mm_unpacklo_epi8(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[j] = _mm_unpacklo_epi8(xmm0[j * 2], xmm0[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm1[2+j] = _mm_unpackhi_epi8(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[2 + j] = _mm_unpackhi_epi8(xmm0[j * 2], xmm0[j * 2 + 1]);
     }
     /* Shuffle 2-byte words */
     for (j = 0; j < 2; j++) {
       /* Compute the low 32 bytes */
-      xmm0[j] = _mm_unpacklo_epi16(xmm1[j*2], xmm1[j*2+1]);
+      xmm0[j] = _mm_unpacklo_epi16(xmm1[j * 2], xmm1[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm0[2+j] = _mm_unpackhi_epi16(xmm1[j*2], xmm1[j*2+1]);
+      xmm0[2 + j] = _mm_unpackhi_epi16(xmm1[j * 2], xmm1[j * 2 + 1]);
     }
     /* Store the result vectors in proper order */
     _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (0 * sizeof(__m128i))), xmm0[0]);
@@ -323,10 +321,9 @@ unshuffle4_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for unshuffling a buffer for a type size of 8 bytes. */
 static void
 unshuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 8;
-  size_t i;
+                const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 8;
+  int32_t i;
   int j;
   __m128i xmm0[8], xmm1[8];
 
@@ -339,23 +336,23 @@ unshuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
     /* Shuffle bytes */
     for (j = 0; j < 4; j++) {
       /* Compute the low 32 bytes */
-      xmm1[j] = _mm_unpacklo_epi8(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[j] = _mm_unpacklo_epi8(xmm0[j * 2], xmm0[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm1[4+j] = _mm_unpackhi_epi8(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[4 + j] = _mm_unpackhi_epi8(xmm0[j * 2], xmm0[j * 2 + 1]);
     }
     /* Shuffle 2-byte words */
     for (j = 0; j < 4; j++) {
       /* Compute the low 32 bytes */
-      xmm0[j] = _mm_unpacklo_epi16(xmm1[j*2], xmm1[j*2+1]);
+      xmm0[j] = _mm_unpacklo_epi16(xmm1[j * 2], xmm1[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm0[4+j] = _mm_unpackhi_epi16(xmm1[j*2], xmm1[j*2+1]);
+      xmm0[4 + j] = _mm_unpackhi_epi16(xmm1[j * 2], xmm1[j * 2 + 1]);
     }
     /* Shuffle 4-byte dwords */
     for (j = 0; j < 4; j++) {
       /* Compute the low 32 bytes */
-      xmm1[j] = _mm_unpacklo_epi32(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[j] = _mm_unpacklo_epi32(xmm0[j * 2], xmm0[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm1[4+j] = _mm_unpackhi_epi32(xmm0[j*2], xmm0[j*2+1]);
+      xmm1[4 + j] = _mm_unpackhi_epi32(xmm0[j * 2], xmm0[j * 2 + 1]);
     }
     /* Store the result vectors in proper order */
     _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (0 * sizeof(__m128i))), xmm1[0]);
@@ -369,13 +366,80 @@ unshuffle8_sse2(uint8_t* const dest, const uint8_t* const src,
   }
 }
 
+/* Routine optimized for unshuffling a buffer for a type size of 12 bytes. */
+/* Based on the 16-byte implementation */
+static void
+unshuffle12_sse2(uint8_t* const dest, const uint8_t* const src,
+                 const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 12;
+  int32_t i;
+  int j;
+  __m128i xmm1[16], xmm2[16];
+
+  __m128i mask = _mm_set_epi8( 0x0, 0x0, 0x0, 0x0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff);
+
+  for (i = 0; i < vectorizable_elements; i += sizeof(__m128i)) {
+    /* Load 16 elements (256 bytes) into 16 XMM registers. */
+    const uint8_t* const src_for_ith_element = src + i;
+    for (j = 0; j < bytesoftype; j++) {
+      xmm1[j] = _mm_loadu_si128((__m128i*)(src_for_ith_element + (j * total_elements)));
+    }
+    /* Shuffle bytes */
+    for (j = 0; j < 8; j++) {
+      /* Compute the low 32 bytes */
+      xmm2[j] = _mm_unpacklo_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
+      /* Compute the hi 32 bytes */
+      xmm2[8 + j] = _mm_unpackhi_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
+    }
+    /* Shuffle 2-byte words */
+    for (j = 0; j < 8; j++) {
+      /* Compute the low 32 bytes */
+      xmm1[j] = _mm_unpacklo_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
+      /* Compute the hi 32 bytes */
+      xmm1[8 + j] = _mm_unpackhi_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
+    }
+    /* Shuffle 4-byte dwords */
+    for (j = 0; j < 8; j++) {
+      /* Compute the low 32 bytes */
+      xmm2[j] = _mm_unpacklo_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
+      /* Compute the hi 32 bytes */
+      xmm2[8 + j] = _mm_unpackhi_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
+    }
+    /* Shuffle 8-byte qwords */
+    for (j = 0; j < 8; j++) {
+      /* Compute the low 32 bytes */
+      xmm1[j] = _mm_unpacklo_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
+      /* Compute the hi 32 bytes */
+      xmm1[8 + j] = _mm_unpackhi_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
+    }
+
+
+    /* Store the result vectors in proper order */
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (0 * 12)), xmm1[0]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (1 * 12)), xmm1[8]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (2 * 12)), xmm1[4]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (3 * 12)), xmm1[12]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (4 * 12)), xmm1[2]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (5 * 12)), xmm1[10]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (6 * 12)), xmm1[6]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (7 * 12)), xmm1[14]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (8 * 12)), xmm1[1]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (9 * 12)), xmm1[9]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (10 * 12)), xmm1[5]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (11 * 12)), xmm1[13]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (12 * 12)), xmm1[3]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (13 * 12)), xmm1[11]);
+    _mm_storeu_si128((__m128i*)(dest + (i * bytesoftype) + (14 * 12)), xmm1[7]);
+    _mm_maskmoveu_si128(xmm1[15], mask, (char *)(dest + (i * bytesoftype) + (15 * 12)));
+  }
+}
+
 /* Routine optimized for unshuffling a buffer for a type size of 16 bytes. */
 static void
 unshuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
-  const size_t vectorizable_elements, const size_t total_elements)
-{
-  static const size_t bytesoftype = 16;
-  size_t i;
+                 const int32_t vectorizable_elements, const int32_t total_elements) {
+  static const int32_t bytesoftype = 16;
+  int32_t i;
   int j;
   __m128i xmm1[16], xmm2[16];
 
@@ -388,30 +452,30 @@ unshuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
     /* Shuffle bytes */
     for (j = 0; j < 8; j++) {
       /* Compute the low 32 bytes */
-      xmm2[j] = _mm_unpacklo_epi8(xmm1[j*2], xmm1[j*2+1]);
+      xmm2[j] = _mm_unpacklo_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm2[8+j] = _mm_unpackhi_epi8(xmm1[j*2], xmm1[j*2+1]);
+      xmm2[8 + j] = _mm_unpackhi_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
     }
     /* Shuffle 2-byte words */
     for (j = 0; j < 8; j++) {
       /* Compute the low 32 bytes */
-      xmm1[j] = _mm_unpacklo_epi16(xmm2[j*2], xmm2[j*2+1]);
+      xmm1[j] = _mm_unpacklo_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm1[8+j] = _mm_unpackhi_epi16(xmm2[j*2], xmm2[j*2+1]);
+      xmm1[8 + j] = _mm_unpackhi_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
     }
     /* Shuffle 4-byte dwords */
     for (j = 0; j < 8; j++) {
       /* Compute the low 32 bytes */
-      xmm2[j] = _mm_unpacklo_epi32(xmm1[j*2], xmm1[j*2+1]);
+      xmm2[j] = _mm_unpacklo_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm2[8+j] = _mm_unpackhi_epi32(xmm1[j*2], xmm1[j*2+1]);
+      xmm2[8 + j] = _mm_unpackhi_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
     }
     /* Shuffle 8-byte qwords */
     for (j = 0; j < 8; j++) {
       /* Compute the low 32 bytes */
-      xmm1[j] = _mm_unpacklo_epi64(xmm2[j*2], xmm2[j*2+1]);
+      xmm1[j] = _mm_unpacklo_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
       /* Compute the hi 32 bytes */
-      xmm1[8+j] = _mm_unpackhi_epi64(xmm2[j*2], xmm2[j*2+1]);
+      xmm1[8 + j] = _mm_unpackhi_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
     }
 
     /* Store the result vectors in proper order */
@@ -437,10 +501,9 @@ unshuffle16_sse2(uint8_t* const dest, const uint8_t* const src,
 /* Routine optimized for unshuffling a buffer for a type size larger than 16 bytes. */
 static void
 unshuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const orig,
-  const size_t vectorizable_elements, const size_t total_elements, const size_t bytesoftype)
-{
-  size_t i;
-  const size_t vecs_per_el_rem = bytesoftype % sizeof(__m128i);
+                       const int32_t vectorizable_elements, const int32_t total_elements, const int32_t bytesoftype) {
+  int32_t i;
+  const int32_t vecs_per_el_rem = bytesoftype % (int32_t)sizeof(__m128i);
 
   int j;
   uint8_t* dest_with_offset;
@@ -448,9 +511,10 @@ unshuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const orig,
 
   /* The unshuffle loops are inverted (compared to shuffle_tiled16_sse2)
      to optimize cache utilization. */
-  size_t offset_into_type;
+  int32_t offset_into_type;
   for (offset_into_type = 0; offset_into_type < bytesoftype;
-    offset_into_type += (offset_into_type == 0 && vecs_per_el_rem > 0 ? vecs_per_el_rem : sizeof(__m128i))) {
+       offset_into_type += (offset_into_type == 0 &&
+           vecs_per_el_rem > 0 ? vecs_per_el_rem : (int32_t)sizeof(__m128i))) {
     for (i = 0; i < vectorizable_elements; i += sizeof(__m128i)) {
       /* Load the first 128 bytes in 16 XMM registers */
       const uint8_t* const src_for_ith_element = orig + i;
@@ -460,30 +524,30 @@ unshuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const orig,
       /* Shuffle bytes */
       for (j = 0; j < 8; j++) {
         /* Compute the low 32 bytes */
-        xmm2[j] = _mm_unpacklo_epi8(xmm1[j*2], xmm1[j*2+1]);
+        xmm2[j] = _mm_unpacklo_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
         /* Compute the hi 32 bytes */
-        xmm2[8+j] = _mm_unpackhi_epi8(xmm1[j*2], xmm1[j*2+1]);
+        xmm2[8 + j] = _mm_unpackhi_epi8(xmm1[j * 2], xmm1[j * 2 + 1]);
       }
       /* Shuffle 2-byte words */
       for (j = 0; j < 8; j++) {
         /* Compute the low 32 bytes */
-        xmm1[j] = _mm_unpacklo_epi16(xmm2[j*2], xmm2[j*2+1]);
+        xmm1[j] = _mm_unpacklo_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
         /* Compute the hi 32 bytes */
-        xmm1[8+j] = _mm_unpackhi_epi16(xmm2[j*2], xmm2[j*2+1]);
+        xmm1[8 + j] = _mm_unpackhi_epi16(xmm2[j * 2], xmm2[j * 2 + 1]);
       }
       /* Shuffle 4-byte dwords */
       for (j = 0; j < 8; j++) {
         /* Compute the low 32 bytes */
-        xmm2[j] = _mm_unpacklo_epi32(xmm1[j*2], xmm1[j*2+1]);
+        xmm2[j] = _mm_unpacklo_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
         /* Compute the hi 32 bytes */
-        xmm2[8+j] = _mm_unpackhi_epi32(xmm1[j*2], xmm1[j*2+1]);
+        xmm2[8 + j] = _mm_unpackhi_epi32(xmm1[j * 2], xmm1[j * 2 + 1]);
       }
       /* Shuffle 8-byte qwords */
       for (j = 0; j < 8; j++) {
         /* Compute the low 32 bytes */
-        xmm1[j] = _mm_unpacklo_epi64(xmm2[j*2], xmm2[j*2+1]);
+        xmm1[j] = _mm_unpacklo_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
         /* Compute the hi 32 bytes */
-        xmm1[8+j] = _mm_unpackhi_epi64(xmm2[j*2], xmm2[j*2+1]);
+        xmm1[8 + j] = _mm_unpackhi_epi64(xmm2[j * 2], xmm2[j * 2 + 1]);
       }
 
       /* Store the result vectors in proper order */
@@ -510,51 +574,50 @@ unshuffle16_tiled_sse2(uint8_t* const dest, const uint8_t* const orig,
 
 /* Shuffle a block.  This can never fail. */
 void
-blosc_internal_shuffle_sse2(const size_t bytesoftype, const size_t blocksize,
-                            const uint8_t* const _src, uint8_t* const _dest) {
-  const size_t vectorized_chunk_size = bytesoftype * sizeof(__m128i);
+shuffle_sse2(const int32_t bytesoftype, const int32_t blocksize,
+             const uint8_t *_src, uint8_t *_dest) {
+  const int32_t vectorized_chunk_size = bytesoftype * (int32_t)sizeof(__m128i);
   /* If the blocksize is not a multiple of both the typesize and
      the vector size, round the blocksize down to the next value
      which is a multiple of both. The vectorized shuffle can be
      used for that portion of the data, and the naive implementation
      can be used for the remaining portion. */
-  const size_t vectorizable_bytes = blocksize - (blocksize % vectorized_chunk_size);
-  const size_t vectorizable_elements = vectorizable_bytes / bytesoftype;
-  const size_t total_elements = blocksize / bytesoftype;
+  const int32_t vectorizable_bytes = blocksize - (blocksize % vectorized_chunk_size);
+  const int32_t vectorizable_elements = vectorizable_bytes / bytesoftype;
+  const int32_t total_elements = blocksize / bytesoftype;
 
   /* If the block size is too small to be vectorized,
      use the generic implementation. */
   if (blocksize < vectorized_chunk_size) {
-    blosc_internal_shuffle_generic(bytesoftype, blocksize, _src, _dest);
+    shuffle_generic(bytesoftype, blocksize, _src, _dest);
     return;
   }
 
   /* Optimized shuffle implementations */
-  switch (bytesoftype)
-  {
-  case 2:
-    shuffle2_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 4:
-    shuffle4_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 8:
-    shuffle8_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 16:
-    shuffle16_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  default:
-    if (bytesoftype > sizeof(__m128i)) {
-      shuffle16_tiled_sse2(_dest, _src, vectorizable_elements, total_elements, bytesoftype);
-    }
-    else {
-      /* Non-optimized shuffle */
-      blosc_internal_shuffle_generic(bytesoftype, blocksize, _src, _dest);
-      /* The non-optimized function covers the whole buffer,
-         so we're done processing here. */
-      return;
-    }
+  switch (bytesoftype) {
+    case 2:
+      shuffle2_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 4:
+      shuffle4_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 8:
+      shuffle8_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 16:
+      shuffle16_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    default:
+      if (bytesoftype > (int32_t)sizeof(__m128i)) {
+        shuffle16_tiled_sse2(_dest, _src, vectorizable_elements, total_elements, bytesoftype);
+      }
+      else {
+        /* Non-optimized shuffle */
+        shuffle_generic(bytesoftype, blocksize, _src, _dest);
+        /* The non-optimized function covers the whole buffer,
+           so we're done processing here. */
+        return;
+      }
   }
 
   /* If the buffer had any bytes at the end which couldn't be handled
@@ -567,52 +630,53 @@ blosc_internal_shuffle_sse2(const size_t bytesoftype, const size_t blocksize,
 
 /* Unshuffle a block.  This can never fail. */
 void
-blosc_internal_unshuffle_sse2(const size_t bytesoftype, const size_t blocksize,
-                              const uint8_t* const _src, uint8_t* const _dest) {
-  const size_t vectorized_chunk_size = bytesoftype * sizeof(__m128i);
+unshuffle_sse2(const int32_t bytesoftype, const int32_t blocksize,
+               const uint8_t *_src, uint8_t *_dest) {
+  const int32_t vectorized_chunk_size = bytesoftype * (int32_t)sizeof(__m128i);
   /* If the blocksize is not a multiple of both the typesize and
      the vector size, round the blocksize down to the next value
      which is a multiple of both. The vectorized unshuffle can be
      used for that portion of the data, and the naive implementation
      can be used for the remaining portion. */
-  const size_t vectorizable_bytes = blocksize - (blocksize % vectorized_chunk_size);
-  const size_t vectorizable_elements = vectorizable_bytes / bytesoftype;
-  const size_t total_elements = blocksize / bytesoftype;
-
+  const int32_t vectorizable_bytes = blocksize - (blocksize % vectorized_chunk_size);
+  const int32_t vectorizable_elements = vectorizable_bytes / bytesoftype;
+  const int32_t total_elements = blocksize / bytesoftype;
 
   /* If the block size is too small to be vectorized,
      use the generic implementation. */
   if (blocksize < vectorized_chunk_size) {
-    blosc_internal_unshuffle_generic(bytesoftype, blocksize, _src, _dest);
+    unshuffle_generic(bytesoftype, blocksize, _src, _dest);
     return;
   }
 
   /* Optimized unshuffle implementations */
-  switch (bytesoftype)
-  {
-  case 2:
-    unshuffle2_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 4:
-    unshuffle4_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 8:
-    unshuffle8_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  case 16:
-    unshuffle16_sse2(_dest, _src, vectorizable_elements, total_elements);
-    break;
-  default:
-    if (bytesoftype > sizeof(__m128i)) {
-      unshuffle16_tiled_sse2(_dest, _src, vectorizable_elements, total_elements, bytesoftype);
-    }
-    else {
-      /* Non-optimized unshuffle */
-      blosc_internal_unshuffle_generic(bytesoftype, blocksize, _src, _dest);
-      /* The non-optimized function covers the whole buffer,
-         so we're done processing here. */
-      return;
-    }
+  switch (bytesoftype) {
+    case 2:
+      unshuffle2_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 4:
+      unshuffle4_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 8:
+      unshuffle8_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 12:
+      unshuffle12_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    case 16:
+      unshuffle16_sse2(_dest, _src, vectorizable_elements, total_elements);
+      break;
+    default:
+      if (bytesoftype > (int32_t)sizeof(__m128i)) {
+        unshuffle16_tiled_sse2(_dest, _src, vectorizable_elements, total_elements, bytesoftype);
+      }
+      else {
+        /* Non-optimized unshuffle */
+        unshuffle_generic(bytesoftype, blocksize, _src, _dest);
+        /* The non-optimized function covers the whole buffer,
+           so we're done processing here. */
+        return;
+      }
   }
 
   /* If the buffer had any bytes at the end which couldn't be handled
@@ -623,4 +687,20 @@ blosc_internal_unshuffle_sse2(const size_t bytesoftype, const size_t blocksize,
   }
 }
 
-#endif /* !defined(__SSE2__) */
+const bool is_shuffle_sse2 = true;
+
+#else /* defined(__SSE2__) */
+
+const bool is_shuffle_sse2 = false;
+
+void shuffle_sse2(const int32_t bytesoftype, const int32_t blocksize,
+                  const uint8_t *_src, uint8_t *_dest) {
+  abort();
+}
+
+void unshuffle_sse2(const int32_t bytesoftype, const int32_t blocksize,
+                    const uint8_t *_src, uint8_t *_dest) {
+  abort();
+}
+
+#endif /* defined(__SSE2__) */
