@@ -3,8 +3,9 @@
 
   Unit tests for Blosc API.
 
-  Creation date: 2010-06-07
-  Author: Francesc Alted <francesc@blosc.org>
+  Copyright (c) 2021  Blosc Development Team <blosc@blosc.org>
+  https://blosc.org
+  License: BSD 3-Clause (see LICENSE.txt)
 
   See LICENSE.txt for details about copyright and rights to use.
 **********************************************************************/
@@ -14,128 +15,96 @@
 int tests_run = 0;
 
 /* Global vars */
-void *src, *srccpy, *dest, *dest2;
-size_t nbytes, cbytes;
+void* src, * srccpy, * dest, * dest2;
+int nbytes, cbytes;
 int clevel = 3;
 int doshuffle = 1;
-size_t typesize = 4;
-size_t size = 1*MB;
+int typesize = 4;
+int size = 1 * MB;
 
 
-
-static const char *test_cbuffer_sizes(void) {
+static char* test_cbuffer_sizes(void) {
   size_t nbytes_, cbytes_, blocksize;
 
-  blosc_cbuffer_sizes(dest, &nbytes_, &cbytes_, &blocksize);
+  blosc1_cbuffer_sizes(dest, &nbytes_, &cbytes_, &blocksize);
   mu_assert("ERROR: nbytes incorrect(1)", nbytes == size);
-  mu_assert("ERROR: nbytes incorrect(2)", nbytes_ == nbytes);
-  mu_assert("ERROR: cbytes incorrect", cbytes == cbytes_);
+  mu_assert("ERROR: nbytes incorrect(2)", nbytes_ == (size_t)nbytes);
+  mu_assert("ERROR: cbytes incorrect", cbytes_ == (size_t)cbytes);
   mu_assert("ERROR: blocksize incorrect", blocksize >= 128);
   return 0;
 }
 
-static const char *test_get_complib_info(void) {
-  char *complib, *version;
-  int clibcode;
-
-  clibcode = blosc_get_complib_info("blosclz", &complib, &version);
-  mu_assert("ERROR: complib incorrect", strcmp(complib, "BloscLZ") == 0);
-  mu_assert("ERROR: clibcode incorrect", clibcode == 0);
-  free(complib);
-  free(version);
-  clibcode = blosc_get_complib_info("non-existing", &complib, &version);
-  mu_assert("ERROR: complib should be NULL", complib == NULL);
-  mu_assert("ERROR: clibcode incorrect", clibcode == -1);
-  clibcode = blosc_get_complib_info("blosclz", NULL, NULL);
-  mu_assert("ERROR: clibcode incorrect", clibcode == 0);
-  clibcode = blosc_get_complib_info("blosclz", NULL, &version);
-  mu_assert("ERROR: clibcode incorrect", clibcode == 0);
-  clibcode = blosc_get_complib_info("blosclz", &complib, NULL);
-  mu_assert("ERROR: complib incorrect", strcmp(complib, "BloscLZ") == 0);
-  mu_assert("ERROR: clibcode incorrect", clibcode == 0);
-  return 0;
-}
-
-static const char *test_cbuffer_metainfo(void) {
+static char* test_cbuffer_metainfo(void) {
   size_t typesize_;
   int flags;
 
-  blosc_cbuffer_metainfo(dest, &typesize_, &flags);
-  mu_assert("ERROR: typesize incorrect", typesize_ == typesize);
+  blosc1_cbuffer_metainfo(dest, &typesize_, &flags);
+  mu_assert("ERROR: typesize incorrect", typesize_ == (size_t)typesize);
   mu_assert("ERROR: shuffle incorrect", (flags & BLOSC_DOSHUFFLE) == doshuffle);
   return 0;
 }
 
 
-static const char *test_cbuffer_versions(void) {
+static char* test_cbuffer_versions(void) {
   int version_;
   int versionlz_;
 
-  blosc_cbuffer_versions(dest, &version_, &versionlz_);
-  mu_assert("ERROR: version incorrect", version_ == BLOSC_VERSION_FORMAT);
+  blosc2_cbuffer_versions(dest, &version_, &versionlz_);
+  mu_assert("ERROR: version incorrect", version_ == BLOSC2_VERSION_FORMAT);
   mu_assert("ERROR: versionlz incorrect", versionlz_ == BLOSC_BLOSCLZ_VERSION_FORMAT);
   return 0;
 }
 
 
-static const char *test_cbuffer_complib(void) {
-  const char *complib;
+static char* test_cbuffer_complib(void) {
+  const char* complib;
 
-  complib = blosc_cbuffer_complib(dest);
+  complib = blosc2_cbuffer_complib(dest);
   mu_assert("ERROR: complib incorrect", strcmp(complib, "BloscLZ") == 0);
   return 0;
 }
 
+static char *test_nthreads(void) {
+  int16_t nthreads;
 
-static const char *test_nthreads(void) {
-  int nthreads;
-
-  nthreads = blosc_set_nthreads(4);
+  nthreads = blosc2_set_nthreads(4);
   mu_assert("ERROR: set_nthreads incorrect", nthreads == 1);
-  nthreads = blosc_get_nthreads();
+  nthreads = blosc2_get_nthreads();
   mu_assert("ERROR: get_nthreads incorrect", nthreads == 4);
   return 0;
 }
 
-static const char *test_blocksize(void) {
+static char *test_blocksize(void) {
   int blocksize;
 
-  blocksize = blosc_get_blocksize();
+  blocksize = blosc1_get_blocksize();
   mu_assert("ERROR: get_blocksize incorrect", blocksize == 0);
 
-  blosc_set_blocksize(4096);
-  blocksize = blosc_get_blocksize();
+  blosc1_set_blocksize(4096);
+  blocksize = blosc1_get_blocksize();
   mu_assert("ERROR: get_blocksize incorrect", blocksize == 4096);
   return 0;
 }
 
-static char *test_set_splitmode() {
-  blosc_set_splitmode(BLOSC_AUTO_SPLIT);
-  return 0;
-}
 
-
-static const char *all_tests(void) {
+static char* all_tests(void) {
   mu_run_test(test_cbuffer_sizes);
-  mu_run_test(test_get_complib_info);
   mu_run_test(test_cbuffer_metainfo);
   mu_run_test(test_cbuffer_versions);
   mu_run_test(test_cbuffer_complib);
   mu_run_test(test_nthreads);
   mu_run_test(test_blocksize);
-  mu_run_test(test_set_splitmode);
   return 0;
 }
 
 #define BUFFER_ALIGN_SIZE   8
 
-int main(int argc, char **argv) {
-  const char *result;
+int main(void) {
+  char* result;
 
-  printf("STARTING TESTS for %s", argv[0]);
-
-  blosc_init();
-  blosc_set_nthreads(1);
+  install_blosc_callback_test(); /* optionally install callback test */
+  blosc2_init();
+  blosc2_set_nthreads(1);
 
   /* Initialize buffers */
   src = blosc_test_malloc(BUFFER_ALIGN_SIZE, size);
@@ -146,10 +115,10 @@ int main(int argc, char **argv) {
   memcpy(srccpy, src, size);
 
   /* Get a compressed buffer */
-  cbytes = blosc_compress(clevel, doshuffle, typesize, size, src, dest, size);
+  cbytes = blosc1_compress(clevel, doshuffle, typesize, size, src, dest, size);
 
   /* Get a decompressed buffer */
-  nbytes = blosc_decompress(dest, dest2, size);
+  nbytes = blosc1_decompress(dest, dest2, size);
 
   /* Run all the suite */
   result = all_tests();
@@ -166,8 +135,7 @@ int main(int argc, char **argv) {
   blosc_test_free(dest);
   blosc_test_free(dest2);
 
-  blosc_destroy();
+  blosc2_destroy();
 
   return result != 0;
 }
-
